@@ -1,6 +1,6 @@
 #==============================================================================
-# LibWeb::Themes::Default -- a component of LibWeb--a Perl library/toolkit for
-#                            building World Wide Web applications.
+# LibWeb::Themes::Default -- Default HTML widgets and theme for libweb
+#                            applications
 
 package LibWeb::Themes::Default;
 
@@ -21,20 +21,26 @@ package LibWeb::Themes::Default;
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #=============================================================================
 
-# For debugging purposes.  Should be commented out in production release.
+# $Id: Default.pm,v 1.6 2000/07/19 20:31:57 ckyc Exp $
 
+$VERSION = '0.02';
+
+#-#############################
 # Use standard libraries.
 use strict;
 use vars qw($VERSION @ISA);
+use Carp;
 
+#-#############################
 # Use custom libraries.
 require LibWeb::Core;
 
-$VERSION = '0.01';
+#-#############################
+# Inheritance.
 @ISA = qw(LibWeb::Core);
 
-#=================================================================================
-# Constructor
+#-#############################
+# Methods.
 sub new {
     #
     # Params: $class [, $rc_file]
@@ -48,16 +54,28 @@ sub new {
     my ($class, $Class, $self);
     $class = shift;
     $Class = ref($class) || $class;
-
+    
     # Inherit instance variables from the base class.
     $self = $Class->SUPER::new(shift);
     bless($self, $Class);
 }
 
-sub DESTROY {
-    #Destructor: performs cleanup when this object is not being
-    #referenced any more.  For example, disconnect a database
-    #connection, filehandle...etc.
+sub DESTROY {}
+
+sub _parse_content {
+    my ($ref, @content_display);
+    my $content = $_[0];
+    
+    eval {
+	foreach (@$content) {
+	    $ref = ref($_);
+	    if ( $ref eq 'SCALAR' ) { push(@content_display, $$_); }
+	    elsif ( $ref eq 'ARRAY' ) { push(@content_display, @$_); }
+	    else { push(@content_display, $_); }
+	}
+    };
+    croak "$@" if ($@);
+    return \@content_display;
 }
 
 sub table {
@@ -74,29 +92,24 @@ sub table {
     # Return a SCALAR ref. to a  formatted table in HTML format.
     #
     my ($self, $content, $bg_color, $align, $valign, $width,
-	@content_display, $ref);
+	$content_display);
     $self = shift;
     ($content, $bg_color, $align, $valign, $width)
       = $self->rearrange( ['CONTENT', 'BG_COLOR', 'ALIGN', 'VALIGN', 'WIDTH'], @_ );
-
+    
     $content ||= [' '];
     $bg_color ||= $self->{SITE_BG_COLOR};
     $align ||= 'left';
     $valign ||= 'top';
     $width ||= '100%';
-
-    foreach (@$content) {
-	$ref = ref($_);
-	if ( $ref eq 'SCALAR' ) { push(@content_display, $$_); }
-	elsif ( $ref eq 'ARRAY' ) { push(@content_display, @$_); }
-        else { push(@content_display, $_); }
-    }
-
+    
+    $content_display = _parse_content($content);
+    
 #<!-- start HTML for table -->
-return \<<HTML;
+    return \<<HTML;
 <table border=0 cellpadding=0 cellspacing=0 width="$width" bgcolor="$bg_color">
 <Tr><td align="$align" valign="$valign">
-@content_display
+@$content_display
 </td></Tr>
 </table>
 
@@ -131,7 +144,7 @@ sub titled_table {
     my ($self,
 	$title, $content, $title_space, $title_align, $title_bg_color,
 	$title_txt_color, $bg_color, $align, $valign, $cellpadding, $width,
-	$title_spacer, @content_display, $ref);
+	$title_spacer, $content_display);
     $self = shift;
     ($title, $content, $title_space, $title_align, $title_bg_color,
      $title_txt_color, $bg_color, $align, $valign, $cellpadding, $width)
@@ -139,7 +152,7 @@ sub titled_table {
 			   'TITLE_BG_COLOR', 'TITLE_TXT_COLOR', 'BG_COLOR',
 			   'ALIGN', 'VALIGN', 'CELLPADDING', 'WIDTH'],
 			  @_ );
-
+    
     $title ||= " ";
     $content ||= [' '];
     $title_align ||= 'center';
@@ -147,8 +160,8 @@ sub titled_table {
 	$title_space = ( uc($title_align) eq 'CENTER' ) ? 0 : 2;
     }
     $title_spacer = '&nbsp;' x $title_space;
-    if ( uc($title_align) eq 'RIGHT' ) { $title = '<b>'.$title.'<b>'.$title_spacer; }
-    else { $title = $title_spacer.'<b>'.$title.'<b>'; }
+    if ( uc($title_align) eq 'RIGHT' ) { $title = '<b>'.$title.'</b>'.$title_spacer; }
+    else { $title = $title_spacer.'<b>'.$title.'</b>'; }
     $title_bg_color ||= $self->{SITE_1ST_COLOR};
     $title_txt_color ||= $self->{SITE_BG_COLOR};
     $bg_color ||= $self->{SITE_BG_COLOR};
@@ -156,16 +169,11 @@ sub titled_table {
     $valign ||= 'top';
     $cellpadding ||= 1;
     $width ||= '100%';
-
-    foreach (@$content) {
-	$ref = ref($_);
-	if ( $ref eq 'SCALAR' ) { push(@content_display, $$_); }
-	elsif ( $ref eq 'ARRAY' ) { push(@content_display, @$_); }
-        else { push(@content_display, $_); }
-    }
-
+    
+    $content_display = _parse_content($content);
+    
 #<!-- start HTML for titled_table -->
-return \<<HTML;
+    return \<<HTML;
 <table border=0 cellpadding=0 cellspacing=0 width="$width" bgcolor="$bg_color">
 
 <Tr><td>
@@ -173,12 +181,12 @@ return \<<HTML;
 <Tr><td align="$title_align" bgcolor="$title_bg_color">
 <font color="$title_txt_color">$title</font>
 </td></Tr></table>
-</td><Tr>
+</td></Tr>
 
 <Tr><td>
 <table bgcolor="$bg_color" cellpadding="$cellpadding" cellspacing=0 width="100%" border=0>
 <Tr><td align="$align" valign="$valign">
-@content_display
+@$content_display
 </td></Tr></table>
 </td></Tr>
 
@@ -189,6 +197,10 @@ HTML
 }
 
 sub titled_table_enlighted {
+    shift->enlighted_titled_table(@_);
+}
+
+sub enlighted_titled_table {
     #
     # Params: -title=>, -content=>, [ -title_space=>, -title_align=>,
     #                               -title_bg_color=>,
@@ -218,7 +230,7 @@ sub titled_table_enlighted {
 	$title, $content, $title_space, $title_align, $title_bg_color,
 	$title_txt_color, $title_border_color, $bg_color, $align, $valign,
 	$cellpadding, $width,
-	$title_spacer, @content_display, $ref);
+	$title_spacer, $content_display);
     $self = shift;
     ($title, $content, $title_space, $title_align, $title_bg_color,
      $title_txt_color, $title_border_color, $bg_color, $align, $valign,
@@ -227,7 +239,7 @@ sub titled_table_enlighted {
 			   'TITLE_BG_COLOR', 'TITLE_TXT_COLOR', 'TITLE_BORDER_COLOR',
 			   'BG_COLOR', 'ALIGN', 'VALIGN', 'CELLPADDING', 'WIDTH'],
 			  @_ );
-
+    
     $title ||= " ";
     $content ||= [' '];
     $title_align ||= 'center';
@@ -245,16 +257,11 @@ sub titled_table_enlighted {
     $valign ||= 'top';
     $cellpadding ||= 1;
     $width ||= '100%';
-
-    foreach (@$content) {
-	$ref = ref($_);
-	if ( $ref eq 'SCALAR' ) { push(@content_display, $$_); }
-	elsif ( $ref eq 'ARRAY' ) { push(@content_display, @$_); }
-        else { push(@content_display, $_); }
-    }
-
+    
+    $content_display = _parse_content($content);
+    
 #<!-- start HTML for titled_table_enlighted -->
-return \<<HTML;
+    return \<<HTML;
 <table border=0 cellpadding=0 cellspacing=0 width="$width" bgcolor="$bg_color">
 
 <Tr><td>
@@ -267,12 +274,12 @@ return \<<HTML;
 </table>
 </td></Tr>
 </table>
-</td><Tr>
+</td></Tr>
 
 <Tr><td>
 <table bgcolor="$bg_color" cellpadding="$cellpadding" cellspacing=0 width="100%" border=0>
 <Tr><td align="$align" valign="$valign">
-@content_display
+@$content_display
 </td></Tr>
 </table>
 </td></Tr>
@@ -290,47 +297,42 @@ sub bordered_table {
     #
     # -content must be an ARRAY ref. to elements which are
     #  scalar/SCALAR ref./ARRAY ref. to plain HTML.
-    # -border_color default is SITE_1ST_COLOR.
+    # -border_color default is SITE_LIQUID_COLOR2.
     # -bg_color default is if SITE_BG_COLOR.
     # -align is the content's horizontal alignment; default is `left'.
     # -valign is the content's vertical alignment; default is `top'.
     # -cellpadding is the distance between the content and the `border' of table,
-    #  default is 1.
+    #  default is 2.
     # -width default is 100%.
     #
     # Return a SCALAR ref. to a formatted table in HTML format.
     #
     my ($self,
 	$content, $border_color, $bg_color, $align, $valign, $cellpadding, $width,
-	@content_display, $ref);
+	$content_display);
     $self = shift;
     ($content, $border_color, $bg_color, $align, $valign, $cellpadding, $width)
       = $self->rearrange( ['CONTENT', 'BORDER_COLOR', 'BG_COLOR', 'ALIGN', 'VALIGN',
 			   'CELLPADDING', 'WIDTH'], @_ );
-
+    
     $content ||= [' '];
-    $border_color ||= $self->{SITE_1ST_COLOR};
+    $border_color ||= $self->{SITE_LIQUID_COLOR2};
     $bg_color ||= $self->{SITE_BG_COLOR};
     $align ||= 'left';
     $valign ||= 'top';
-    $cellpadding ||= 1;
+    $cellpadding ||= 2;
     $width ||= '100%';
-
-    foreach (@$content) {
-	$ref = ref($_);
-	if ( $ref eq 'SCALAR' ) { push(@content_display, $$_); }
-	elsif ( $ref eq 'ARRAY' ) { push(@content_display, @$_); }
-        else { push(@content_display, $_); }
-    }
-
+    
+    $content_display = _parse_content($content);
+    
 #<!-- start HTML for bordered_table -->
-return \<<HTML;
+    return \<<HTML;
 <table border=0 cellpadding=1 cellspacing=0 bgcolor="$border_color" width="$width">
 
 <Tr><td>
 <table bgcolor="$bg_color" cellpadding="$cellpadding" cellspacing=0 width="100%" border=0>
 <Tr><td align="$align" valign="$valign" bgcolor="$bg_color">
-@content_display
+@$content_display
 </td></Tr></table>
 </td></Tr>
 
@@ -341,6 +343,10 @@ HTML
 }
 
 sub titled_bordered_table {
+    shift->bordered_titled_table(@_);
+}
+
+sub bordered_titled_table {
     #
     # Params: -title=>, -content=>, [ -title_space=>, -title_align=>,
     #                               -border_color=>,
@@ -368,7 +374,7 @@ sub titled_bordered_table {
     my ($self,
 	$title, $content, $title_space, $title_align, $border_color,
 	$title_txt_color, $bg_color, $align, $valign, $cellpadding, $width,
-	$title_spacer, @content_display, $ref);
+	$title_spacer, $content_display);
     $self = shift;
     ($title, $content, $title_space, $title_align, $border_color,
      $title_txt_color, $bg_color, $align, $valign, $cellpadding, $width)
@@ -376,7 +382,7 @@ sub titled_bordered_table {
 			   'BORDER_COLOR', 'TITLE_TXT_COLOR',
 			   'BG_COLOR', 'ALIGN', 'VALIGN', 'CELLPADDING', 'WIDTH'],
 			  @_ );
-
+    
     $title ||= " ";
     $content ||= [' '];
     $title_align ||= 'center';
@@ -384,8 +390,8 @@ sub titled_bordered_table {
 	$title_space = ( uc($title_align) eq 'CENTER' ) ? 0 : 2;
     }
     $title_spacer = '&nbsp;' x $title_space;
-    if ( uc($title_align) eq 'RIGHT' ) { $title = '<b>'.$title.'<b>'.$title_spacer; }
-    else { $title = $title_spacer.'<b>'.$title.'<b>'; }
+    if ( uc($title_align) eq 'RIGHT' ) { $title = '<b>'.$title.'</b>'.$title_spacer; }
+    else { $title = $title_spacer.'<b>'.$title.'</b>'; }
     $border_color ||= $self->{SITE_1ST_COLOR};
     $title_txt_color ||= $self->{SITE_BG_COLOR};
     $bg_color ||= $self->{SITE_BG_COLOR};
@@ -393,16 +399,11 @@ sub titled_bordered_table {
     $valign ||= 'top';
     $cellpadding ||= 1;
     $width ||= '100%';
-
-    foreach (@$content) {
-	$ref = ref($_);
-	if ( $ref eq 'SCALAR' ) { push(@content_display, $$_); }
-	elsif ( $ref eq 'ARRAY' ) { push(@content_display, @$_); }
-        else { push(@content_display, $_); }
-    }
-
+    
+    $content_display = _parse_content($content);
+    
 #<!-- start HTML for titled_bordered_table -->
-return \<<HTML;
+    return \<<HTML;
 <table border=0 cellpadding=1 cellspacing=0 bgcolor="$border_color" width="$width">
 
 <Tr><td>
@@ -415,7 +416,7 @@ return \<<HTML;
 <Tr><td>
 <table bgcolor="$bg_color" cellpadding="$cellpadding" cellspacing=0 width="100%" border=0>
 <Tr><td align="$align" valign="$valign" bgcolor="$bg_color">
-@content_display
+@$content_display
 </td></Tr></table>
 </td></Tr>
 
@@ -425,17 +426,153 @@ HTML
 #<!-- end HTML for the titled_bordered_table -->
 }
 
+sub tabber {
+    #
+    # Params:
+    #
+    # -tabs=>, -active=> [, -active_color=>, -fade_color=>,
+    #                       -tab_padding=>, -tab_width=>,
+    #                       -gap_width=>, -total_width=>,
+    #                       -base=>, -base_height=>, -left_stub=>,
+    #                       -right_stub=>, -left_stub_align=>,
+    #                       -right_stub_align=>, -left_stub_width=>,
+    #                       -right_stub_width=> ]
+    #
+    # Pre:
+    # - -tabs is an ARRAY reference to scalar/SCALAR reference
+    # - -active is a number indicating which tab is currently active (count from 0)
+    # - -active_color is the color for the active tab; default is SITE_1ST_COLOR
+    # - -fade_color is the color for non-active tabs; default is SITE_LIQUID_COLOR3
+    # - -tab_padding is the distance between a tab's content to its border, either
+    #   in pixel or percentage; default is 0
+    # - -tab_width is the width of each tab, either in pixel or percentage; default
+    #   is ( total_width / (2 * number of tabs) )
+    # - -gap_width is the distance between two adjacent tabs, either in pixel or
+    #   percentage; default if 1%
+    # - -total_width is the width of the whole tabber, either in pixel or percentage;
+    #   default is 100%
+    # - -base is a SCALAR reference to HTML to be put under the tabs
+    # - -base_height is the height of -base, either in pixel or percentage; default
+    #   is 5.  You do not want to specify this if you specify -base
+    # - base_align default is `left'
+    # - -left_stub is a SCALAR reference to HTML to be put to the left of the tabs
+    # - -right_stub is a SCALAR reference to HTML to be put to the right of the tabs
+    # - -left_stub_align is 'left', 'center' or 'right'; default is 'left'
+    # - -right_stub_align is 'left', 'center' or 'right'; default is 'right'
+    # - -left_stub_width is either in pixel or percentage, you may want to specify
+    #   this if you specify -left_stub
+    # - -right_stub_width is either in pixel or percentage
+    #
+    # Post:
+    #
+    # Return a SCALAR ref. to a formatted tabbing navigation bar in HTML format.
+    #
+    my (
+	$self,
+	$tabs, $active, $active_color, $fade_color, $tab_padding, $tab_width,
+	$gap_width, $total_width, $base, $base_height, $base_align, $left_stub,
+	$right_stub, $left_stub_align, $right_stub_align, $left_stub_width,
+	$right_stub_width,
+	$num_tabs, $num_elements, $tabs_html, $count
+       );
+    $self = shift;
+    (
+     $tabs, $active, $active_color, $fade_color, $tab_padding, $tab_width,
+     $gap_width, $total_width, $base, $base_height, $base_align, $left_stub,
+     $right_stub, $left_stub_align, $right_stub_align, $left_stub_width,
+     $right_stub_width
+    ) = $self->rearrange(
+			 [
+			  'TABS','ACTIVE','ACTIVE_COLOR','FADE_COLOR','TAB_PADDING',
+			  'TAB_WIDTH','GAP_WIDTH','TOTAL_WIDTH','BASE','BASE_HEIGHT',
+			  'BASE_ALIGN','LEFT_STUB','RIGHT_STUB','LEFT_STUB_ALIGN',
+			  'RIGHT_STUB_ALIGN','LEFT_STUB_WIDTH','RIGHT_STUB_WIDTH'
+			 ], @_
+			);
+
+    $active ||= 0;
+    $active_color ||= $self->{SITE_1ST_COLOR};
+    $fade_color ||= $self->{SITE_LIQUID_COLOR3};
+    $tab_padding ||= '0';
+    $gap_width = '1%' unless ( defined $gap_width );
+    $total_width ||= '100%';
+    $base_height ||= '5';
+    $base_align ||= 'left';
+    $base ||= \ ("<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\"><Tr><td height=\"$base_height\"></td></Tr></table>");
+    $tabs_html = '';
+    
+    # Determine how many columns the base should span.
+    eval { $num_tabs = scalar @{$tabs}; };
+    croak "$@" if ($@);    
+    $num_elements = 2 * $num_tabs + ( ($left_stub || $left_stub_width) ? 1 : 0 ) + 1;
+
+    # Determine the width for individual tab.
+    if ( $total_width =~ m:(.*)\%$: ) {
+	$tab_width ||= int ( $1 / (2*$num_tabs) ) . '%';
+    } else {
+	$tab_width ||= int ( $total_width / (2*$num_tabs) );
+    }
+    
+    eval {
+
+	# Prepend the left stub.
+	if ($left_stub || $left_stub_width) {
+	    $left_stub ||= \('');
+	    $left_stub_align ||= 'left';
+	    $left_stub_width ||=  ( $tab_width =~ m:(.*)\%$: ) ?
+	                          int( (100 - $1 * $num_tabs)/2 ).'%' :
+				  int( $total_width - $num_tabs * $tab_width);
+	    $tabs_html .=
+	      "<td width=\"$left_stub_width\" align=\"$left_stub_align\">&nbsp;$${left_stub}&nbsp;</td>\n";
+	}
+	
+	# Create the tabs.
+	$count = 0;
+	foreach ( @{$tabs} ) {	    
+	    my $the_tab;
+	    my $color = ($count == $active) ? $active_color : $fade_color;
+	    if ( ref $_ ) {
+		$the_tab = $$_;
+	    } else {
+		$the_tab = $_;
+	    }
+	    
+	    $tabs_html .=
+	      "<td align=\"center\" bgcolor=\"$color\" width=\"$tab_width\"><table bgcolor=\"$color\" border=\"0\" width=\"100%\" cellspacing=\"0\" cellpadding=\"$tab_padding\"><Tr align=\"center\"><td>".
+		"&nbsp;${the_tab}&nbsp;</td></Tr></table></td>";
+	    $tabs_html .= "<td width=\"$gap_width\">&nbsp;</td>\n" if ($gap_width);
+
+	    $count++;	    
+	}
+
+	# Append the right stub.
+	$right_stub ||= \('');
+	$right_stub_align ||= 'right';
+	$right_stub_width ||= ( 100 - $num_tabs ) . '%';
+	$tabs_html .= "<td width=\"$right_stub_width\" align=\"$right_stub_align\">&nbsp;$${right_stub}&nbsp;</td>\n";
+    };
+    croak "$@" if ($@);
+
+return \<<HTML;
+<table border="0" width="$total_width" cellspacing="0" cellpadding="0">
+<Tr>$tabs_html</Tr>
+<Tr><td width="100%" colspan="$num_elements">
+<table border="0" cellspacing="0" cellpadding="0" width="100%"><Tr><td bgcolor="$active_color" align="$base_align">$$base</td></Tr></table>
+</td></Tr>
+</table>
+HTML
+}
+
 1;
 __DATA__
 
 1;
 __END__
 
-=pod
-
 =head1 NAME
 
-LibWeb::Themes::Default - DEFAULT HTML THEME FOR LIBWEB APPLICATIONS
+LibWeb::Themes::Default - Default HTML widgets and theme for libweb
+applications
 
 =head1 SUPPORTED PLATFORMS
 
@@ -470,8 +607,15 @@ LibWeb::Core
   use LibWeb::Themes::Default;
   my $theme = new LibWeb::Themes::Default();
 
-  my $navigation_bar =
-      $theme->table( -content => [ $nav_bar ] );
+  my $tabs = [
+               '<A HREF="/"><IMG SRC="/img/icon_home.png">Home</A>',
+               '<A HREF="/reports">Featured reports</A>', 
+               '<A HREF="/shopping">Best buy</A>',
+               '<A HREF="/log">Login</A>'
+             ];
+
+  my $tabbed_navigation_bar =
+      $theme->tabber( -tabs => $tabs, -active => 2 );
 
   my $stock_quotes =
       $theme->bordered_table(
@@ -479,7 +623,7 @@ LibWeb::Core
                             );
 
   my $weather =
-      $theme->titled_bordered_table(
+      $theme->bordered_titled_table(
                                      -title => "Today's weather",
                                      -content => [ $fetched_weather ]
                                    );
@@ -491,7 +635,7 @@ LibWeb::Core
                           );
 
   my $news =
-      $theme->titled_table_enlighted(
+      $theme->enlighted_titled_table(
                                       -title => "Today's news",
                                       -content => [ $fetched_news ]
                                     );
@@ -505,18 +649,17 @@ This class provides several common table templates for HTML display.
 This allows Web application designers focus their efforts on the logic
 of programs; while Web page designers spend their efforts on
 customizing this default by ISAing this class, say
-LibWeb::Themes::Futuristic.
+LibWeb::Themes::Futuristic, and writing different themes.  A sample
+MyTheme.pm is included in the distribution for your hacking pleasure.
 
 The current version of LibWeb::Themes::Default is available at
 
    http://libweb.sourceforge.net
-   ftp://libweb.sourceforge/pub/libweb
 
 Several LibWeb applications (LEAPs) have be written, released and
 are available at
 
    http://leaps.sourceforge.net
-   ftp://leaps.sourceforge.net/pub/leaps
 
 =head1 TYPOGRAPHICAL CONVENTIONS AND TERMINOLOGY
 
@@ -526,34 +669,344 @@ information.  Method's parameters in square brackets means optional.
 
 =head1 DESCRIPTION
 
-B<table()>
+B<bordered_table()>
 
 Params:
 
-  -content=>, [ -bg_color=>, -align=>, -valign=>, -width=> ]
+  -content=> [, -border_color=>, -bg_color=>, align=>,
+  -valign=>, -cellpadding=>, -width=> ]
+
+Pre:
 
 =over 2
 
 =item *
 
--content must be an ARRAY reference to elements which are
+C<-content> must be an ARRAY reference to elements which are
+scalars/SCALAR references/ARRAY references to plain HTML.
+
+=item *
+
+C<-border_color> default is SITE_LIQUID_COLOR2,
+
+=item *
+
+C<-bg_color> default is SITE_BG_COLOR,
+
+=item *
+
+C<-align> is the content's horizontal alignment; default is `left',
+
+=item *
+
+C<-valign> is the content's vertical alignment; default is `top',
+
+=item *
+
+C<-cellpadding> is the distance between the content and the `border'
+of table, default is 2,
+
+=item *
+
+C<-width> default is 100%.
+
+=back
+
+Post:
+
+=over 2
+
+=item *
+
+Return a SCALAR reference to a formatted table in HTML.
+
+=back
+
+B<bordered_titled_table()>
+
+Params:
+
+  -title=>, -content=> [, -title_space=>, -title_align=>,
+  -border_color=>, -title_txt_color=>, -bg_color=>, -align=>,
+  -valign=>, -cellpadding=>, -width=> ]
+
+Pre:
+
+=over 2
+
+=item *
+
+-title is a scalar,
+
+=item *
+
+C<-content> must be an ARRAY reference to elements which are
+scalars/SCALAR references/ARRAY references to plain HTML.
+
+=item *
+
+C<-title_space> is the space (&nbsp;) prepended before (if
+C<-title_align> is `left') or append after (if C<-title_align> is
+`right') the title; default is 2, i.e.  `&nbsp;&nbsp;'.  If
+C<-title_align> is `center', -title_space is always 0,
+
+=item *
+
+C<-title_align> default is `center',
+
+=item *
+
+C<-border_color> default is SITE_1ST_COLOR,
+
+=item *
+
+C<-title_txt_color> default is SITE_BG_COLOR,
+
+=item *
+
+C<-bg_color> default is SITE_BG_COLOR,
+
+=item *
+
+C<-align> is the content's horizontal alignment; default is `left',
+
+=item *
+
+C<-valign> is the content's vertical alignment; default is `top',
+
+=item *
+
+C<-cellpadding> is the distance between the content and the `border'
+of table, default is 1,
+
+=item *
+
+C<-width> default is 100%.
+
+=back
+
+Post:
+
+=over 2
+
+=item *
+
+Return a SCALAR reference to a formatted table in HTML.
+
+=back
+
+B<enlighted_titled_table()>
+
+Params:
+
+  -title=>, -content=> [, -title_space=>, -title_align=>,
+  -title_bg_color=>, -title_txt_color=>, -title_border_color=>,
+  -bg_color=>, -align=>, -valign=>, -cellpadding=>, -width=> ]
+
+Pre:
+
+=over 2
+
+=item *
+
+C<-title> is a scalar,
+
+=item *
+
+C<-content> must be an ARRAY reference to elements which are
 scalars/SCALAR references/ARRAY references to plain HTML,
 
 =item *
 
--bg_color default is SITE_BG_COLOR,
+C<-title_space> is the space (&nbsp;) prepended before (if
+C<-title_align> is `left') or append after (if C<-title_align> is
+`right') the title; default is 2, i.e. `&nbsp;&nbsp;'.  It is always 0
+if C<-title_align> is `center',
 
 =item *
 
--align is the content's horizontal alignment; default is `left',
+C<-title_align> default is `center',
 
 =item *
 
--valign is the content's vertical alignment; default is `top',
+C<-title_bg_color> default is SITE_LIQUID_COLOR3,
 
 =item *
 
--width default is 100%.
+C<-title_txt_color> default is SITE_LIQUID_COLOR5,
+
+=item *
+
+C<-title_border_color> default is SITE_LIQUID_COLOR5,
+
+=item *
+
+C<-bg_color> default is SITE_BG_COLOR,
+
+=item *
+
+C<-align> is the content's horizontal alignment; default is `left',
+
+=item *
+
+C<-valign> is the content's vertical alignment; default is `top',
+
+=item *
+
+C<-cellpadding> is the distance between the content and the `border'
+of table, default is 1.
+
+=item *
+
+C<-width> default is 100%.
+
+=back
+
+Post:
+
+=over 2
+
+=item *
+
+Return a SCALAR reference to a formatted table in HTML.
+
+=back
+
+B<tabber()>
+
+Params:
+
+  -tabs=>, -active=> [, -active_color=>, -fade_color=>,
+  -tab_padding=>, -tab_width=>, -gap_width=>, -total_width=>,
+  -base=>, -base_height=>, -base_align, -left_stub=>,
+  -right_stub=>, -left_stub_align=>, -right_stub_align=>,
+  -left_stub_width=>, -right_stub_width=> ]
+
+Pre:
+
+=over 2
+
+=item *
+
+C<-tabs> is an ARRAY reference to scalar or SCALAR reference
+
+=item *
+
+C<-active> is a number indicating which tab is currently active (count
+from 0)
+
+=item *
+
+C<-active_color> is the color for the active tab; default is
+SITE_1ST_COLOR
+
+=item *
+
+C<-fade_color> is the color for non-active tabs; default is
+SITE_LIQUID_COLOR3
+
+=item *
+
+C<-tab_padding> is the distance between a tab's content to its border,
+either in pixel or percentage; default is 0
+
+=item *
+
+C<-tab_width> is the width of each tab, either in pixel or percentage;
+default is ( total_width / (2 * number of tabs) )
+
+=item *
+
+C<-gap_width> is the distance between two adjacent tabs, either in pixel
+or percentage; default if 1%
+
+=item *
+
+C<-total_width> is the width of the whole tabber, either in pixel or
+percentage; default is 100%
+
+=item *
+
+C<-base> is a SCALAR reference to HTML to be put under the tabs
+
+=item *
+
+C<-base_height> is the height of -base, either in pixel or percentage;
+default is 5.  You do not want to specify this if you specify C<-base>
+
+=item *
+
+C<-base_align> default is `left'
+
+=item *
+
+C<-left_stub> is a SCALAR reference to HTML to be put to the left of
+the tabs
+
+=item *
+
+C<-right_stub> is a SCALAR reference to HTML to be put to the right of
+the tabs
+
+=item *
+
+C<-left_stub_align> is 'left', 'center' or 'right'; default is 'left'
+
+=item *
+
+C<-right_stub_align> is 'left', 'center' or 'right'; default is 'right'
+
+=item *
+
+C<-left_stub_width> is either in pixel or percentage, you may want to
+specify this if you specify C<-left_stub>
+
+=item *
+
+C<-right_stub_width> is either in pixel or percentage
+
+=back
+
+Post:
+
+=over 2
+
+=item *
+
+Return a SCALAR reference to a formatted tabbing navigation bar in HTML format.
+
+=back
+
+B<table()>
+
+Params:
+
+  -content=> [, -bg_color=>, -align=>, -valign=>, -width=> ]
+
+Pre:
+
+=over 2
+
+=item *
+
+C<-content> must be an ARRAY reference to elements which are
+scalars/SCALAR references/ARRAY references to plain HTML,
+
+=item *
+
+C<-bg_color> default is SITE_BG_COLOR,
+
+=item *
+
+C<-align> is the content's horizontal alignment; default is `left',
+
+=item *
+
+C<-valign> is the content's vertical alignment; default is `top',
+
+=item *
+
+C<-width> default is 100%.
 
 =back
 
@@ -571,212 +1024,8 @@ B<titled_table()>
 
 Params:
 
-  -title=>, -content=>, [ -title_space=>, -title_align=>,
+  -title=>, -content=> [, -title_space=>, -title_align=>,
   -title_bg_color=>, -title_txt_color=>, -bg_color=>, align=>,
-  valign=>, -cellpadding=>, -width=> ]
-
-Pre:
-
-=over 2
-
-=item *
-
--title is a scalar,
-
-=item *
-
--content must be an ARRAY reference to elements which are
-scalars/SCALAR references/ARRAY references to plain HTML,
-
-=item *
-
--title_space is the space (&nbsp;) prepended before (if -title_align
-is `left') or append after (if -title_align is `right') the title.
-It's always 0 if -title_align is `center'.  Default is 2,
-i.e. `&nbsp;&nbsp;',
-
-=item *
-
--title_align default is `center',
-
-=item *
-
--title_bg_color default is SITE_1ST_COLOR,
-
-=item *
-
--title_txt_color default is SITE_BG_COLOR,
-
-=item *
-
--bg_color default is SITE_BG_COLOR,
-
-=item *
-
--align is the content's horizontal alignment; default is `left',
-
-=item *
-
--valign is the content's vertical alignment; default is `top',
-
-=item *
-
--cellpadding is the distance between the content and the `border' of
-table, default is 1.
-
-=item *
-
--width default is 100%.
-
-=back
-
-Post:
-
-=over 2
-
-=item *
-
-Return a SCALAR reference to a formatted table in HTML.
-
-=back
-
-B<titled_table_enlighted()>
-
-Params:
-
-  -title=>, -content=>, [ -title_space=>, -title_align=>,
-  -title_bg_color=>, -title_txt_color=>, -title_border_color=>,
-  -bg_color=>, -align=>, -valign=>,
-  -cellpadding=>, -width=> ]
-
-Pre:
-
-=over 2
-
-=item *
-
--title is a scalar,
-
-=item *
-
--content must be an ARRAY reference to elements which are
-scalars/SCALAR references/ARRAY references to plain HTML,
-
-=item *
-
--title_space is the space (&nbsp;) prepended before (if -title_align
-is `left') or append after (if -title_align is `right') the title;
-default is 2, i.e. `&nbsp;&nbsp;'.  It's always 0 if -title_align if
-`center',
-
-=item *
-
--title_align default is `center',
-
-=item *
-
--title_bg_color default is SITE_LIQUID_COLOR3,
-
-=item *
-
--title_txt_color default is SITE_LIQUID_COLOR5,
-
-=item *
-
--title_border_color default is SITE_LIQUID_COLOR5,
-
-=item *
-
--bg_color default is SITE_BG_COLOR,
-
-=item *
-
--align is the content's horizontal alignment; default is `left',
-
-=item *
-
--valign is the content's vertical alignment; default is `top',
-
-=item *
-
--cellpadding is the distance between the content and the `border' of
-table, default is 1.
-
-=item *
-
--width default is 100%.
-
-=back
-
-Post:
-
-=over 2
-
-=item *
-
-Return a SCALAR reference to a formatted table in HTML.
-
-=back
-
-B<bordered_table()>
-
-Params:
-
-  -content=>, [ -border_color=>, -bg_color=>, align=>,
-  -valign=>, -cellpadding=>, -width=> ]
-
-Pre:
-
-=over 2
-
-=item *
-
--content must be an ARRAY reference to elements which are
-scalars/SCALAR references/ARRAY references to plain HTML.
-
-=item *
-
--border_color default is SITE_1ST_COLOR,
-
-=item *
-
--bg_color default is SITE_BG_COLOR,
-
-=item *
-
--align is the content's horizontal alignment; default is `left',
-
-=item *
-
--valign is the content's vertical alignment; default is `top',
-
-=item *
-
--cellpadding is the distance between the content and the `border' of
-table, default is 1,
-
-=item *
-
--width default is 100%.
-
-=back
-
-Post:
-
-=over 2
-
-=item *
-
-Return a SCALAR reference to a formatted table in HTML.
-
-=back
-
-B<titled_bordered_table()>
-
-Params:
-
-  -title=>, -content=>, [ -title_space=>, -title_align=>,
-  -border_color=>, -title_txt_color=>, -bg_color=>, -align=>,
   -valign=>, -cellpadding=>, -width=> ]
 
 Pre:
@@ -789,48 +1038,48 @@ Pre:
 
 =item *
 
--content must be an ARRAY reference to elements which are
-scalars/SCALAR references/ARRAY references to plain HTML.
+C<-content> must be an ARRAY reference to elements which are
+scalars/SCALAR references/ARRAY references to plain HTML,
 
 =item *
 
--title_space is the space (&nbsp;) prepended before (if -title_align
-is `left') or append after (if -title_align is `right') the title;
-default is 2, i.e.  `&nbsp;&nbsp;'.  If -title_align is `center',
--title_space is always 0,
+C<-title_space> is the space (&nbsp;) prepended before (if
+C<-title_align> is `left') or append after (if C<-title_align> is
+`right') the title.  It is always 0 if C<-title_align> is `center'.
+Default is 2, i.e. `&nbsp;&nbsp;',
 
 =item *
 
--title_align default is `center',
+C<-title_align> default is `center',
 
 =item *
 
--border_color default is SITE_1ST_COLOR,
+C<-title_bg_color> default is SITE_1ST_COLOR,
 
 =item *
 
--title_txt_color default is SITE_BG_COLOR,
+C<-title_txt_color> default is SITE_BG_COLOR,
 
 =item *
 
--bg_color default is SITE_BG_COLOR,
+C<-bg_color> default is SITE_BG_COLOR,
 
 =item *
 
--align is the content's horizontal alignment; default is `left',
+C<-align> is the content's horizontal alignment; default is `left',
 
 =item *
 
--valign is the content's vertical alignment; default is `top',
+C<-valign> is the content's vertical alignment; default is `top',
 
 =item *
 
--cellpadding is the distance between the content and the `border' of
-table, default is 1,
+C<-cellpadding> is the distance between the content and the `border' of
+table, default is 1.
 
 =item *
 
--width default is 100%.
+C<-width> default is 100%.
 
 =back
 
@@ -854,7 +1103,6 @@ Return a SCALAR reference to a formatted table in HTML.
 
 =head1 CREDITS
 
-
 =head1 BUGS
 
 This release does not provide a lot of table templates and only a
@@ -865,6 +1113,8 @@ http://libweb.sourceforge.net.
 =head1 SEE ALSO
 
 L<LibWeb::HTML::Error>, L<LibWeb::HTML::Site>,
-L<LibWeb::HTML::Default>.
+L<LibWeb::HTML::Default>
 
 =cut
+#  LocalWords:  bg content's txt uc enlighted tabber tab's num int colspan png
+#  LocalWords:  ISAing Futuristic MyTheme
